// MacでUS配列かつWindowsでJIS配列として認識されている時にどちらでもUS配列として使うキーマップ

#include "planck.h"
#include "action_layer.h"
#include "keymap_jp.h"    // qmk_firmware-master/quantum/keymap_extras/keymap_jp.h

extern keymap_config_t keymap_config;

// レイヤー
enum planck_layers {
  _MAC,             // Mac用デフォルトレイヤー    （ US配列で認識）
  _WIN_JIS,         // Windows用デフォルトレイヤー（JIS配列で認識）
  _LOWER,           // Lower (Mac)
  _RAISE,           // Raise (Mac)
  _RAISEWIN,        // Raise (Windows)
  _FUNC1,           // Function 1      移動、マウス操作、レイヤートグル
  _ADJUST           // Adjust          設定変更など
};

// 特殊な動作のキーコード
enum planck_keycodes {
  MAC = SAFE_RANGE, // デフォルトレイヤーをMac用に切替
  WIN_JIS,          // デフォルトレイヤーをWindows用に切替
  SFT_JQT,          // タップでJISの「'」  ホールドで右Shift  (Windows)
  WN_SCLN,          // タップでJISの「;」  シフトでJISの「:」 (Windows)
  TGL_RIS,          // トグルでRaiseレイヤーに切り替え        (Mac & Windows)
  M_MYID,           // マクロ (stringを出力)
  M_PASS,           // マクロ (stringを出力)
  JROGUE            // マクロ (stringを出力)
};

// キーコード
#define FN1_ESC LT(_FUNC1,KC_ESC)     // タップでESC        ホールドでFunction 1レイヤーon
#define RSFT_QT MT(MOD_RSFT,KC_QUOT)  // タップで「'」      ホールドで右Shift       (Mac)
#define LOW_ES  LT(_LOWER,KC_LANG2)   // タップで英数       ホールドでLower         (Mac)
#define RIS_KN  LT(_RAISE,KC_LANG1)   // タップでかな       ホールドでRaise         (Mac)
#define LOW_MH  LT(_LOWER,KC_MHEN)    // タップで無変換     ホールドでLower         (Windows)
#define RIS_HK  LT(_RAISEWIN,KC_HENK) // タップで変換       ホールドでRaise_Win_JIS (Windows)
#define CTL_ZH  CTL_T(KC_ZKHK)        // タップで半角/全角  ホールドで左Control     (Windows)
#define ADJUST  MO(_ADJUST)           // ホールドでAdjustレイヤーをon
#define MY_VOLD LALT(LSFT(KC_VOLD))   // 細かいボリュームダウン                     (Mac)
#define MY_VOLU LALT(LSFT(KC_VOLU))   // 細かいボリュームアップ                     (Mac)
#define GUI_LCB LGUI(KC_LCBR)         // ブラウザなどのタブ移動ショートカット       (Mac)
#define GUI_RCB LGUI(KC_RCBR)         // ブラウザなどのタブ移動ショートカット       (Mac)
#define GUI_F1  LGUI(KC_F1)           // ウインドウ表示切り替えショートカット       (Mac)
#define ALT_PSC LALT(KC_PSCR)         // Alt + PrintScreen                          (Windows)
#define WN_CAPS S(KC_CAPS)            // Caps Lock                                  (Windows)
#define TGL_LOW TG(_LOWER)            // トグルでLowerレイヤーに切り替え

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

/* Mac用デフォルトレイヤー    （ US配列で認識）
 * ,-----------------------------------------------------------------------------------.
 * | Tab  |   Q  |   W  |   E  |   R  |   T  |   Y  |   U  |   I  |   O  |   P  | Bksp |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |Esc/FN|   A  |   S  |   D  |   F  |   G  |   H  |   J  |   K  |   L  |   ;  |Enter |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |Shift |   Z  |   X  |   C  |   V  |   B  |   N  |   M  |   ,  |   .  |   /  |'/SFT |
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * | Ctrl |Adjust| Alt  | GUI  |英数/L|    Space    |かな/R| Left | Down |  Up  |Right |
 * `-----------------------------------------------------------------------------------'
 */
[_MAC] = {
  {KC_TAB,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,    KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_BSPC},
  {FN1_ESC, KC_A,    KC_S,    KC_D,    KC_F,    KC_G,    KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_ENT},
  {KC_LSFT, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, RSFT_QT},
  {KC_LCTL, ADJUST,  KC_LALT, KC_LGUI, LOW_ES,  KC_SPC,  KC_SPC,  RIS_KN,  KC_LEFT, KC_DOWN, KC_UP,   KC_RGHT}
},

/* Windows用デフォルトレイヤー（JIS配列で認識）
 * ,-----------------------------------------------------------------------------------.
 * | Tab  |   Q  |   W  |   E  |   R  |   T  |   Y  |   U  |   I  |   O  |   P  | Bksp |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |Esc/FN|   A  |   S  |   D  |   F  |   G  |   H  |   J  |   K  |   L  |   ;  |Enter |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |Shift |   Z  |   X  |   C  |   V  |   B  |   N  |   M  |   ,  |   .  |   /  |'/SFT |
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * |ZH/Ctl|Adjust| Alt  | Ctrl |無変/L|    Space    |変換/R| Left | Down |  Up  |Right |
 * `-----------------------------------------------------------------------------------'
 */
[_WIN_JIS] = {
  {KC_TAB,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,    KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_BSPC},
  {FN1_ESC, KC_A,    KC_S,    KC_D,    KC_F,    KC_G,    KC_H,    KC_J,    KC_K,    KC_L,    WN_SCLN, KC_ENT},
  {KC_LSFT, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, SFT_JQT},
  {CTL_ZH,  ADJUST,  KC_LALT, KC_LCTL, LOW_MH,  KC_SPC,  KC_SPC,  RIS_HK,  KC_LEFT, KC_DOWN, KC_UP,   KC_RGHT}
},

/* Lower
 * ,-----------------------------------------------------------------------------------.
 * |      |   1  |   2  |   3  |   4  |   5  |   6  |   7  |   8  |   9  |   0  | Bksp |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |Esc/FN|  F1  |  F2  |  F3  |  F4  |  F5  |  F6  |   4  |   5  |   6  |   -  |      |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |      |  F7  |  F8  |  F9  |  F10 |  F11 |  F12 |   1  |   2  |   3  |   +  |   *  |
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * |      |      |      |      |      |             |   0  |   ,  |   .  |   /  |   =  |
 * `-----------------------------------------------------------------------------------'
 */
[_LOWER] = {
  {_______, KC_1,    KC_2,    KC_3,    KC_4,    KC_5,    KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_BSPC},
  {FN1_ESC, KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_4,    KC_5,    KC_6,    KC_PMNS, _______},
  {_______, KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F11,  KC_F12,  KC_1,    KC_2,    KC_3,    KC_PPLS, KC_PAST},
  {_______, _______, _______, _______, _______, _______, _______, KC_0   , KC_COMM, KC_DOT,  KC_PSLS, KC_PEQL}
},

/* Raise (Mac用)
 * ,-----------------------------------------------------------------------------------.
 * |      |   !  |   @  |   #  |   $  |   %  |   ^  |   &  |   *  |   (  |   )  | Bksp |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |      |      | Vol- | Vol+ | Mute |   \  |   `  |   -  |   =  |   [  |   ]  |      |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |CpsLck|      | Prev | Next | Play |   |  |   ~  |   _  |   +  |   {  |   }  |CpsLck|
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * |      |      |      |      |      |             |      |      |      |      |      |
 * `-----------------------------------------------------------------------------------'
 */
[_RAISE] = {
  {_______, KC_EXLM, KC_AT,   KC_HASH, KC_DLR,  KC_PERC, KC_CIRC, KC_AMPR, KC_ASTR, KC_LPRN, KC_RPRN, KC_BSPC},
  {_______, XXXXXXX, MY_VOLD, MY_VOLU, KC_MUTE, KC_BSLS, KC_GRV,  KC_MINS, KC_EQL,  KC_LBRC ,KC_RBRC, _______},
  {KC_CAPS, XXXXXXX, KC_MRWD, KC_MFFD, KC_MPLY, KC_PIPE, KC_TILD, KC_UNDS, KC_PLUS, KC_LCBR, KC_RCBR, KC_CAPS},
  {_______, _______, _______, _______, XXXXXXX, _______, _______, _______, _______, _______, _______, _______}
},

/* Raise (Windows用)
 * ,-----------------------------------------------------------------------------------.
 * |      |   !  |   @  |   #  |   $  |   %  |   ^  |   &  |   *  |   (  |   )  | Bksp |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |      |      | Vol- | Vol+ | Mute |   ¥  |   `  |   -  |   =  |   [  |   ]  |      |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |CpsLck|      | Prev | Next | Play |   |  |   ~  |   _  |   +  |   {  |   }  |CpsLck|
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * |      |      |      |      |      |             |      |      |      |      |      |
 * `-----------------------------------------------------------------------------------'
 */
[_RAISEWIN] = {
  {_______, KC_EXLM, JP_AT,   KC_HASH, KC_DLR,  KC_PERC, JP_CIRC, JP_AMPR, JP_ASTR, JP_LPRN, JP_RPRN, KC_BSPC},
  {_______, XXXXXXX, KC_VOLD, KC_VOLU, KC_MUTE, JP_YEN,  JP_GRV,  KC_MINS, JP_EQL,  JP_LBRC, JP_RBRC, _______},
  {WN_CAPS, XXXXXXX, KC_MPRV, KC_MNXT, KC_MPLY, JP_PIPE, JP_TILD, JP_UNDS, JP_PLUS, JP_LCBR, JP_RCBR, WN_CAPS},
  {_______, _______, _______, _______, XXXXXXX, _______, _______, _______, _______, _______, _______, _______}
},

/* Function 1  移動、マウス操作、レイヤートグル、マクロ  マウスキーの詳細： https://docs.qmk.fm/mouse_keys.html
 * ,-----------------------------------------------------------------------------------.
 * |PrintS|WH-Rt |WH-Up |WH-Dn |WH-Lt |      | End  |PG Dn |PG Up | Home |      |  Del |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |      | M-Lt | M-Dn | M-Up | M-Rt | ⌘F1 | Left | Down |  Up  |Right |Speed1|      |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |      |jrogue|      |  ID  | Pass | ⌘{  | ⌘}  | M-B1 | M-B2 | M-B3 |Speed0|      |
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * |      |      |      |      |TglLow|             |TglRis| End  |Pg Dn |Pg Up | Home |
 * `-----------------------------------------------------------------------------------'
 */
[_FUNC1] = {
  {ALT_PSC, KC_WH_R, KC_WH_U, KC_WH_D, KC_WH_L, XXXXXXX, KC_END,  KC_PGDN, KC_PGUP, KC_HOME, XXXXXXX, KC_DEL},
  {_______, KC_MS_L, KC_MS_D, KC_MS_U, KC_MS_R, GUI_F1,  KC_LEFT, KC_DOWN, KC_UP,   KC_RGHT, KC_ACL1, _______},
  {_______, JROGUE,  XXXXXXX, M_MYID,  M_PASS,  GUI_LCB, GUI_RCB, KC_BTN1, KC_BTN2, KC_BTN3, KC_ACL0, _______},
  {_______, _______, _______, _______, TGL_LOW, _______, _______, TGL_RIS, KC_END,  KC_PGDN, KC_PGUP, KC_HOME}
},

/* Adjust      デフォルトレイヤーや設定の変更など
 * ,-----------------------------------------------------------------------------------.
 * |      |Reset |      |      |      |      |      |      |      |      |      |      |
 * |------+------+------+------+------+-------------+------+------+------+------+------|
 * |Power |      |      |Aud on|Audoff|      |      | Mac  |WinJIS|      |      |      |
 * |------+------+------+------+------+------|------+------+------+------+------+------|
 * |      |Voice-|Voice+|Mus on|Musoff|      |      |BLtogg| BL - | BL + |BLstep|      |
 * |------+------+------+------+------+------+------+------+------+------+------+------|
 * |      |      |      |      |      |             |      |      |      |      |      |
 * `-----------------------------------------------------------------------------------'
 */
[_ADJUST] = {
  {XXXXXXX, RESET,   XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX, XXXXXXX},
  {KC_POWER,XXXXXXX, MU_MOD,  AU_ON,   AU_OFF,  XXXXXXX, XXXXXXX, MAC,     WIN_JIS, XXXXXXX, XXXXXXX, XXXXXXX},
  {XXXXXXX, MUV_DE,  MUV_IN,  MU_ON,   MU_OFF,  XXXXXXX, XXXXXXX, BL_TOGG, BL_DEC,  BL_INC,  BL_STEP, XXXXXXX},
  {_______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______}
},

};

#ifdef AUDIO_ENABLE
  float layer_lock_on_song[][2]  = SONG(LAYER_LOCK_ON_SOUND);   // Layerロック
  float layer_lock_off_song[][2] = SONG(LAYER_LOCK_OFF_SOUND);  // Layerロック解除
  float caps_lock_song[][2]      = SONG(CAPS_LOCK_ON_SOUND);    // Caps Lockキー
  float adjust_on_song[][2]      = SONG(ADJUST_ON_SOUND);       // Adjustキーon
  float adjust_off_song[][2]     = SONG(ADJUST_OFF_SOUND);      // Adjustキーoff
  float push_song[][2]           = SONG(PUSH_SOUND);            // 汎用
#endif

// LED点灯 & Breathing
void led_breathing_on(uint8_t breathingspeed, bool breathing) {
    #ifdef BACKLIGHT_ENABLE
      backlight_set(1);
      #ifdef BACKLIGHT_BREATHING
        breathing_speed_set(breathingspeed);
        if (breathing) {
            breathing_enable();
        } else {
            breathing_pulse();
        }
      #endif
    #endif
}
// LED消す
void led_breathing_off(void) {
    #ifdef BACKLIGHT_ENABLE
      #ifdef BACKLIGHT_BREATHING
        breathing_speed_set(1);
        breathing_self_disable();
      #endif
      backlight_set(0);
    #endif
}

void init_layer(void) {
    layer_off(_LOWER);
    layer_off(_RAISE);
    layer_off(_RAISEWIN);
    led_breathing_off();
}

bool process_record_user(uint16_t keycode, keyrecord_t *record) {

    static uint16_t custom_timer;    
    static uint8_t  prev_shift;
    static bool     lshift = false;
    static bool     rshift = false;
    static uint8_t  dl;
    static uint8_t  raiselayer;
    
  switch (keycode) {
    case MAC:
      if (record->event.pressed) {
        init_layer();
        set_single_persistent_default_layer(_MAC);
      }
      return false;
      break;
    case WIN_JIS:
      if (record->event.pressed) {
        init_layer();
        set_single_persistent_default_layer(_WIN_JIS);
      }
      return false;
      break;
    case TGL_LOW:
      if (record->event.pressed) {
        if(IS_LAYER_ON(_LOWER)) {
            led_breathing_off();
            #ifdef AUDIO_ENABLE
              PLAY_SONG(layer_lock_off_song);
            #endif
        } else {
            led_breathing_on(1, false);
            #ifdef AUDIO_ENABLE
              PLAY_SONG(layer_lock_on_song);
            #endif
        }
      }
      return true;
      break;
    case TGL_RIS:
      if (record->event.pressed) {
        dl = biton32(default_layer_state);
        if(dl == _MAC){
            raiselayer = _RAISE;
        } else if (dl == _WIN_JIS) {
            raiselayer = _RAISEWIN;    
        }
        if IS_LAYER_ON(raiselayer){
            layer_off(raiselayer);
            led_breathing_off();
            #ifdef AUDIO_ENABLE
              PLAY_SONG(layer_lock_off_song);
            #endif
        } else {
            layer_on(raiselayer);
            led_breathing_on(3, true);
            #ifdef AUDIO_ENABLE
              PLAY_SONG(layer_lock_on_song);
            #endif
        }
      }
      return false;
      break;
    case LOW_ES:
    case LOW_MH:
      if (record->event.pressed) {
          if IS_LAYER_ON(_LOWER){
            layer_off(_LOWER);
            #ifdef AUDIO_ENABLE
              PLAY_SONG(layer_lock_off_song);
            #endif
            return false;
            break;
          } else {
            led_breathing_on(1, false);
          }
      } else {
        led_breathing_off();
      }
      return true;
      break;
    case RIS_KN:
    case RIS_HK:
        if (record->event.pressed) {
          if (IS_LAYER_ON(_RAISE) || IS_LAYER_ON(_RAISEWIN)){
            dl = biton32(default_layer_state);
            if(dl == _MAC){
                raiselayer = _RAISE;
            } else if (dl == _WIN_JIS) {
                raiselayer = _RAISEWIN;    
            }
            layer_off(raiselayer);
            #ifdef AUDIO_ENABLE
              PLAY_SONG(layer_lock_off_song);
            #endif
            return false;
            break;
          } else {
            led_breathing_on(3, true);
          }
        } else {
        led_breathing_off();
        }
      return true;
      break;
    case SFT_JQT:
      if (record->event.pressed) {
        custom_timer = timer_read();
        prev_shift = keyboard_report->mods & (MOD_BIT(KC_LSFT) | MOD_BIT(KC_RSFT));
        register_code (KC_RSFT);
      } else {
            if (timer_elapsed (custom_timer) < TAPPING_TERM) {
                if (prev_shift) {
                    register_code(KC_2);
                    unregister_code(KC_2);
                } else {
                    register_code(KC_7);
                    unregister_code(KC_7);
                }
            }
            unregister_code (KC_RSFT);
      }
      return false;
      break;
    case WN_SCLN:
      if (record->event.pressed) {
        lshift = keyboard_report->mods & MOD_BIT(KC_LSFT);
        rshift = keyboard_report->mods & MOD_BIT(KC_RSFT);
        if (lshift || rshift) {
          if (lshift) unregister_code(KC_LSFT);
          if (rshift) unregister_code(KC_RSFT);
          register_code(JP_COLN);
          unregister_code(JP_COLN);
          if (lshift) register_code(KC_LSFT);
          if (rshift) register_code(KC_RSFT);
        } else {
          register_code(JP_SCLN);
          unregister_code(JP_SCLN);
        }
      }
      return false;
      break;
    case KC_CAPS:
    case WN_CAPS:
      if (record->event.pressed) {
        #ifdef AUDIO_ENABLE
          PLAY_SONG(caps_lock_song);
        #endif
      }
      return true;
      break;
    case BL_TOGG:
    case BL_DEC:
    case BL_INC:
    case BL_STEP:
      if (record->event.pressed) {
        #ifdef AUDIO_ENABLE
          PLAY_SONG(push_song);
        #endif
      }
      return true;
      break;
    case M_MYID:
      if (record->event.pressed) {
        SEND_STRING("id_hogehoge\n");
      }
      return false;
      break;
    case M_PASS:
      if (record->event.pressed) {
        SEND_STRING("password_hogehoge\n");
      }
      return false;
      break;
    case JROGUE:
      if (record->event.pressed) {
        SEND_STRING("jrogue");
      }
      return false;
      break;
    case ADJUST:
      if (record->event.pressed) {
        #ifdef AUDIO_ENABLE
          PLAY_SONG(adjust_on_song);
        #endif
      } else {
          #ifdef AUDIO_ENABLE
            PLAY_SONG(adjust_off_song);
          #endif
      }
      return true;
      break;
  }
  return true;
}
